#ifndef FGGL_GFX_PAINT_H
#define FGGL_GFX_PAINT_H

#include <vector>
#include <fggl/math/types.hpp>

namespace fggl::gfx {

    using RadianAngle = float;

    enum class PathType {
        MOVE,
        PATH,
        BAZIER2,
        BAZIER3,
        COLOUR,
        CLOSE
    };

    struct Path2D {
        inline explicit Path2D( math::vec2 start ) : m_points(), m_types() {
            moveTo( start );
        }

        inline void moveTo(math::vec2 pos) {
            m_points.push_back( pos );
            m_types.push_back( PathType::MOVE );
        }

        inline void pathTo(math::vec2 pos) {
            m_points.push_back( pos );
            m_types.push_back( PathType::PATH );
        }

        void bezierTo(math::vec2 cp1, math::vec2 pos);
        void bezierTo(math::vec2 cp1, math::vec2 cp2, math::vec2 pos);

        void arc(math::vec2 center, float radius, RadianAngle startAngle, RadianAngle endAngle, bool ccw);

        void colour(math::vec3 colour) {
            m_colours.push_back( colour );
            m_types.push_back( PathType::COLOUR );
        }

        void close() {
            pathTo( m_points[0] );
            m_types.push_back( PathType::CLOSE );
        }

        std::vector< math::vec2 > m_points;
        std::vector< PathType > m_types;
        std::vector< math::vec3 > m_colours;
    };

    enum class PaintType {
        FILL,
        STROKE
    };

    struct PaintCmd {
        PaintType type;
        Path2D path;
    };

    class Paint {
        public:
            Paint() = default;
            Paint(Paint& paint) = delete;

            inline void fill(Path2D& path) {
                m_cmds.push_back( { PaintType::FILL, path } );
            }

            void stroke(Path2D& path) {
                m_cmds.push_back( { PaintType::STROKE, path } );
            }

            const std::vector< PaintCmd >& cmds() const {
                return m_cmds;
            }

        private:
            std::vector< PaintCmd > m_cmds;
    };

}

#endif
